@@ -1,6 +1,7 @@
 /*
  *   SPDX-FileCopyrightText: 2012 Aleix Pol Gonzalez <aleixpol@blue-systems.com>
  *   SPDX-FileCopyrightText: 2013 Lukas Appelhans <l.appelhans@gmx.de>
+ *   SPDX-FileCopyrightText: 2024 Harald Sitter <sitter@kde.org>
  *
  *   SPDX-License-Identifier: GPL-2.0-only OR GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
  */
@@ -30,7 +31,10 @@ extern "C" {
 
 using namespace Qt::StringLiterals;
 
-const QStringList PackageKitResource::m_topObjects({QStringLiteral("qrc:/qml/DependenciesButton.qml"), QStringLiteral("qrc:/qml/PackageKitPermissions.qml")});
+const QStringList PackageKitResource::s_topObjects({
+    QStringLiteral("qrc:/qml/DependenciesButton.qml"),
+    QStringLiteral("qrc:/qml/PackageKitPermissions.qml"),
+});
 
 PackageKitResource::PackageKitResource(QString packageName, QString summary, PackageKitBackend *parent)
     : AbstractResource(parent)
@@ -38,6 +42,11 @@ PackageKitResource::PackageKitResource(QString packageName, QString summary, Pac
     , m_name(std::move(packageName))
 {
     setObjectName(m_name);
+    connect(this, &AbstractResource::stateChanged, &m_dependencies, &PackageKitDependencies::setDirty);
+    connect(&m_dependencies, &PackageKitDependencies::dependenciesChanged, this, [this] {
+        Q_EMIT dependenciesChanged();
+        Q_EMIT sizeChanged();
+    });
 }
 
 QString PackageKitResource::name() const
@@ -59,12 +68,14 @@ QString PackageKitResource::availablePackageId() const
 {
     // First we check if it's upgradeable and use this version to display
     const QSet<QString> pkgids = backend()->upgradeablePackageId(this);
-    if (!pkgids.isEmpty())
+    if (!pkgids.isEmpty()) {
         return *pkgids.constBegin();
+    }
 
     const auto it = m_packages.constFind(PackageKit::Transaction::InfoAvailable);
-    if (it != m_packages.constEnd())
+    if (it != m_packages.constEnd()) {
         return it->first();
+    }
     return installedPackageId();
 }
 
@@ -97,25 +108,25 @@ QVariant PackageKitResource::icon() const
 }
 
 static QMap<QString, QString> s_translation = {
-    {u"AGPL"_s, u"AGPL-3.0"_s},
-    {u"AGPL3"_s, u"AGPL-3.0"_s},
-    {u"Artistic2.0"_s, u"Artistic-2.0"_s},
-    {u"Apache"_s, u"Apache-2.0"_s},
-    {u"APACHE"_s, u"Apache-2.0"_s},
-    {u"CCPL"_s, u"CC0-1.0"_s},
-    {u"GPL2"_s, u"GPL-2.0"_s},
-    {u"GPL3"_s, u"GPL-3.0"_s},
-    {u"FDL1.2"_s, u"GFDL-1.2-only"_s},
-    {u"FDL1.3"_s, u"GFDL-1.3-only"_s},
-    {u"LGPL"_s, u"LGPL-2.1"_s},
-    {u"LGPL3"_s, u"LGPL-3.0"_s},
-    {u"MPL"_s, u"MPL-1.1"_s},
-    {u"MPL2"_s, u"MPL-2.0"_s},
-    {u"PerlArtistic"_s, u"Artistic-1.0-Perl"_s},
-    {u"PHP"_s, u"PHP-3.01"_s},
-    {u"PSF"_s, u"Python-2.0"_s},
-    {u"RUBY"_s, u"Ruby"_s},
-    {u"ZPL"_s, u"ZPL-2.1"_s},
+    {u"AGPL"_s, u"AGPL-3.0"_s}, //
+    {u"AGPL3"_s, u"AGPL-3.0"_s}, //
+    {u"Artistic2.0"_s, u"Artistic-2.0"_s}, //
+    {u"Apache"_s, u"Apache-2.0"_s}, //
+    {u"APACHE"_s, u"Apache-2.0"_s}, //
+    {u"CCPL"_s, u"CC0-1.0"_s}, //
+    {u"GPL2"_s, u"GPL-2.0"_s}, //
+    {u"GPL3"_s, u"GPL-3.0"_s}, //
+    {u"FDL1.2"_s, u"GFDL-1.2-only"_s}, //
+    {u"FDL1.3"_s, u"GFDL-1.3-only"_s}, //
+    {u"LGPL"_s, u"LGPL-2.1"_s}, //
+    {u"LGPL3"_s, u"LGPL-3.0"_s}, //
+    {u"MPL"_s, u"MPL-1.1"_s}, //
+    {u"MPL2"_s, u"MPL-2.0"_s}, //
+    {u"PerlArtistic"_s, u"Artistic-1.0-Perl"_s}, //
+    {u"PHP"_s, u"PHP-3.01"_s}, //
+    {u"PSF"_s, u"Python-2.0"_s}, //
+    {u"RUBY"_s, u"Ruby"_s}, //
+    {u"ZPL"_s, u"ZPL-2.1"_s}, //
 };
 
 QJsonArray PackageKitResource::licenses()
@@ -176,11 +187,11 @@ QString PackageKitResource::origin() const
         // so we can set the OS name if we see it as origin prefix, and otherwise need to fall back
         // to the origin string.
         int i = dataField.indexOf(QLatin1Char(':'));
-        QString origin = i > 0? dataField.mid(i + 1) : dataField;
+        QString origin = i > 0 ? dataField.mid(i + 1) : dataField;
         if (origin.startsWith(osRelease->id().toLower() + QLatin1Char('-'))) {
             return osRelease->name();
         } else {
-            return origin.isEmpty()? i18n("Unknown Source") : origin;
+            return origin.isEmpty() ? i18n("Unknown Source") : origin;
         }
     }
 
@@ -201,26 +212,29 @@ QString PackageKitResource::section()
 
 AbstractResource::State PackageKitResource::state()
 {
-    if (backend()->isPackageNameUpgradeable(this))
+    if (backend()->isPackageNameUpgradeable(this)) {
         return Upgradeable;
-    else if (m_packages.contains(PackageKit::Transaction::InfoInstalled))
+    } else if (m_packages.contains(PackageKit::Transaction::InfoInstalled)) {
         return Installed;
-    else if (m_packages.contains(PackageKit::Transaction::InfoAvailable))
+    } else if (m_packages.contains(PackageKit::Transaction::InfoAvailable)) {
         return None;
-    else
+    } else {
         return Broken;
+    }
 }
 
 void PackageKitResource::addPackageId(PackageKit::Transaction::Info info, const QString &packageId, bool arch)
 {
     auto oldState = state();
-    if (arch)
+    if (arch) {
         m_packages[info].archPkgIds.append(packageId);
-    else
+    } else {
         m_packages[info].nonarchPkgIds.append(packageId);
+    }
 
-    if (oldState != state())
+    if (oldState != state()) {
         Q_EMIT stateChanged();
+    }
 
     Q_EMIT versionsChanged();
 }
@@ -238,8 +252,9 @@ AbstractResource::Type PackageKitResource::type() const
 void PackageKitResource::fetchDetails()
 {
     const QString pkgid = availablePackageId();
-    if (!m_details.isEmpty() || pkgid.isEmpty())
+    if (!m_details.isEmpty() || pkgid.isEmpty()) {
         return;
+    }
     m_details.insert(QStringLiteral("fetching"), true); // we add an entry so it's not re-fetched.
 
     backend()->fetchDetails(pkgid);
@@ -250,19 +265,17 @@ void PackageKitResource::failedFetchingDetails(PackageKit::Transaction::Error er
     qWarning() << "error fetching details" << error << msg;
 }
 
-void PackageKitResource::setDependenciesCount(int deps)
+QList<PackageKitDependency> PackageKitResource::dependencies()
 {
-    if (deps != m_dependenciesCount) {
-        m_dependenciesCount = deps;
-        Q_EMIT sizeChanged();
-    }
+    return m_dependencies.dependencies();
 }
 
 void PackageKitResource::setDetails(const PackageKit::Details &details)
 {
     const bool ourDetails = details.packageId() == availablePackageId();
-    if (!ourDetails)
+    if (!ourDetails) {
         return;
+    }
 
     if (m_details != details) {
         const auto oldState = state();
@@ -271,11 +284,13 @@ void PackageKitResource::setDetails(const PackageKit::Details &details)
         const auto oldDescription = m_details.description();
         m_details = details;
 
-        if (oldState != state())
+        if (oldState != state()) {
             Q_EMIT stateChanged();
+        }
 
-        if (!backend()->isFetching())
+        if (!backend()->isFetching()) {
             Q_EMIT backend()->resourcesChanged(this, {"size", "homepage", "license"});
+        }
 
         if (oldSize != uint(size())) {
             Q_EMIT sizeChanged();
@@ -312,9 +327,10 @@ void PackageKitResource::fetchUpdateDetails()
 
 static void addIfNotEmpty(const QString &title, const QString &content, QString &where)
 {
-    if (!content.isEmpty())
+    if (!content.isEmpty()) {
         where += QLatin1String("<p><b>") + title + QLatin1String("</b>&nbsp;") + QString(content).replace(QLatin1Char('\n'), QLatin1String("<br />"))
             + QLatin1String("</p>");
+    }
 }
 
 QString PackageKitResource::joinPackages(const QStringList &pkgids, const QString &_sep, const QString &shadowPackage)
@@ -322,10 +338,11 @@ QString PackageKitResource::joinPackages(const QStringList &pkgids, const QStrin
     QStringList ret;
     for (const QString &pkgid : pkgids) {
         const auto pkgname = PackageKit::Daemon::packageName(pkgid);
-        if (pkgname == shadowPackage)
+        if (pkgname == shadowPackage) {
             ret += PackageKit::Daemon::packageVersion(pkgid);
-        else
+        } else {
             ret += i18nc("package-name (version)", "%1 (%2)", pkgname, PackageKit::Daemon::packageVersion(pkgid));
+        }
     }
     const QString sep = _sep.isEmpty() ? i18nc("comma separating package names", ", ") : _sep;
     return ret.join(sep);
@@ -334,8 +351,9 @@ QString PackageKitResource::joinPackages(const QStringList &pkgids, const QStrin
 static QStringList urlToLinks(const QStringList &urls)
 {
     QStringList ret;
-    for (const QString &in : urls)
-        ret += QStringLiteral("<a href='%1'>%1</a>").arg(in);
+    for (const auto &url : urls) {
+        ret += QStringLiteral("<a href='%1'>%1</a>").arg(url);
+    }
     return ret;
 }
 
@@ -396,8 +414,9 @@ void PackageKitResource::updateDetail(const QString &packageID,
     addIfNotEmpty(i18n("Update State:"), PackageKitMessages::updateStateMessage(state), info);
     addIfNotEmpty(i18n("Restart:"), PackageKitMessages::restartMessage(restart), info);
 
-    if (!vendorUrls.isEmpty())
+    if (!vendorUrls.isEmpty()) {
         addIfNotEmpty(i18n("Vendor:"), urlToLinks(vendorUrls).join(QLatin1String(", ")), info);
+    }
 
     Q_EMIT changelogFetched(info);
 }
@@ -409,19 +428,23 @@ PackageKitBackend *PackageKitResource::backend() const
 
 QString PackageKitResource::sizeDescription()
 {
+    auto baseDescription = AbstractResource::sizeDescription();
+
     if (state() != AbstractResource::State::None) {
-        return AbstractResource::sizeDescription();
+        return baseDescription;
     }
 
-    if (m_dependenciesCount < 0) {
+    if (!m_dependencies.hasFetchedDependencies()) {
         fetchDetails();
-        fetchDependencies();
+        updatePackageIdForDependencies();
+        return baseDescription;
     }
 
-    if (m_dependenciesCount <= 0)
-        return AbstractResource::sizeDescription();
-    else
-        return i18np("%2 (plus %1 dependency)", "%2 (plus %1 dependencies)", m_dependenciesCount, AbstractResource::sizeDescription());
+    const auto dependenciesCount = m_dependencies.dependencies().count();
+    if (dependenciesCount == 0) {
+        return baseDescription;
+    }
+    return i18np("%2 (plus %1 dependency)", "%2 (plus %1 dependencies)", dependenciesCount, baseDescription);
 }
 
 QString PackageKitResource::sourceIcon() const
@@ -429,52 +452,31 @@ QString PackageKitResource::sourceIcon() const
     return QStringLiteral("package-x-generic");
 }
 
-void PackageKitResource::fetchDependencies()
+QStringList PackageKitResource::topObjects() const
 {
-    const auto id = isInstalled() ? installedPackageId() : availablePackageId();
-    if (id.isEmpty())
-        return;
-    m_dependenciesCount = 0;
+    return s_topObjects;
+}
 
-    auto packageDependencies = QSharedPointer<QJsonArray>::create();
-
-    auto trans = PackageKit::Daemon::dependsOn(id);
-    connect(trans, &PackageKit::Transaction::errorCode, this, [this](PackageKit::Transaction::Error, const QString &message) {
-        qWarning() << "Transaction error: " << message << sender();
-    });
-    connect(trans,
-            &PackageKit::Transaction::package,
-            this,
-            [packageDependencies](PackageKit::Transaction::Info info, const QString &packageID, const QString &summary) {
-                (*packageDependencies)
-                    .append(QJsonObject{{QStringLiteral("packageName"), PackageKit::Daemon::packageName(packageID)},
-                                        {QStringLiteral("packageInfo"), PackageKitMessages::info(info)},
-                                        {QStringLiteral("packageDescription"), summary}});
-            });
-    connect(trans, &PackageKit::Transaction::finished, this, [this, packageDependencies](PackageKit::Transaction::Exit /*status*/) {
-        std::sort(packageDependencies->begin(), packageDependencies->end(), [](QJsonValue a, QJsonValue b) {
-            const auto objA = a.toObject(), objB = b.toObject();
-            return objA[QLatin1String("packageInfo")].toString() < objB[QLatin1String("packageInfo")].toString()
-                || (objA[QLatin1String("packageInfo")].toString() == objB[QLatin1String("packageInfo")].toString()
-                    && objA[QLatin1String("packageName")].toString() < objB[QLatin1String("packageName")].toString());
-        });
-
-        Q_EMIT dependenciesFound(*packageDependencies);
-        setDependenciesCount(packageDependencies->size());
-    });
+void PackageKitResource::updatePackageIdForDependencies()
+{
+    const auto packageId = isInstalled() ? installedPackageId() : availablePackageId();
+    m_dependencies.setPackageId(packageId);
+    m_dependencies.refresh(); // In case packageId didn't actually change.
 }
 
 bool PackageKitResource::extendsItself() const
 {
     const auto extendsResources = backend()->resourcesByPackageNames<QVector<AbstractResource *>>(extends());
-    if (extendsResources.isEmpty())
+    if (extendsResources.isEmpty()) {
         return false;
+    }
 
     const auto ourPackageNames = allPackageNames();
-    for (auto r : extendsResources) {
-        PackageKitResource *pkres = qobject_cast<PackageKitResource *>(r);
-        if (pkres->allPackageNames() != ourPackageNames)
+    for (auto resource : extendsResources) {
+        auto pkResource = qobject_cast<PackageKitResource *>(resource);
+        if (pkResource->allPackageNames() != ourPackageNames) {
             return false;
+        }
     }
     return true;
 }
@@ -495,3 +497,5 @@ bool PackageKitResource::isCritical() const
 {
     return false;
 }
+
+#include "moc_PackageKitResource.cpp"
