@@ -64,12 +64,21 @@ public Q_SLOTS:
     /* Rebase to the next major version */
     void rebaseToNewVersion();
 
+    /* Called when the Transaction changes state. Mostly to cleanup once it's done  */
+    void transactionStatusChanged(Transaction::Status status);
+
 private:
     /* Once rpm-ostree has effectively stated, registrer ourselves as update
      * driver to the rpm-ostreed daemon to make sure that it does not exit while
      * we are running and then initialize the rest of the backend. */
     void initializeBackend();
 
+    /* Check if a transaction has been started outside of Discover */
+    bool hasExternalTransaction();
+
+    /* Helper to setup a Transaction and connect all signals/slots */
+    void setupTransaction(RpmOstreeTransaction::Operation op, QString arg = {});
+
     /* Called when a new major version is found to setup user facing actions */
     void foundNewMajorVersion(const QString &newMajorVersion);
 
